@@ -19,11 +19,10 @@
  */
 
 #include <config.h>
-
-#include <glib.h>
 #include <stdio.h>
 #include <string.h>
 #include <time.h>
+#include <glib.h>
 #include <libgnomevfs/gnome-vfs.h>
 #include <gw-obex.h>
 
@@ -127,6 +126,8 @@ om_utils_get_parent_path_from_uri (const GnomeVFSURI *uri)
 
 	parent_path = g_strconcat (tmp, "/", NULL);
 
+	g_free (tmp);
+
 	return parent_path;
 }
 
@@ -153,6 +154,10 @@ om_utils_get_path_list_from_uri (const gchar       *cur_dir,
 		new_path = om_utils_get_path_from_uri (uri);
 	}
 
+        if (new_path == NULL) {
+                new_path = g_strdup ("");
+        }
+
 	if (cur_dir && strcmp (new_path, cur_dir) == 0) {
 		/* Same path */
 		g_free (new_path);
@@ -198,6 +203,8 @@ om_utils_get_path_list_from_uri (const gchar       *cur_dir,
 		path_start = ch;
 	}
 
+	g_free (new_path);
+
 	list = g_list_reverse (list);
 
 	return list;
@@ -276,33 +283,51 @@ om_utils_obex_error_to_vfs_result (gint error)
 		break;
 	}
 
-	/* FIXME: OpenObex (and gw-obex) doesn't have a mapping for all useful
-	 * error codes. Catch those like this for now:
-	 */
+	/* Handle the OBEX error values as well: */
 	switch (error) {
 	case OBEX_RSP_BAD_REQUEST: /* 0x40 */
 		/* We get this when trying to move a file on Nokia 6230 and
 		 * HP iPaq, does it mean that it's not a supported action?
 		 */
+		d(g_printerr ("Error: RSP_BAD_REQUEST\n"));
 		return GNOME_VFS_ERROR_NOT_SUPPORTED;
 
+	case OBEX_RSP_UNAUTHORIZED: /* 0x41 */
+		d(g_printerr ("Error: RSP_UNAUTHORIZED\n"));
+		return GNOME_VFS_ERROR_NOT_PERMITTED;
+
 	case OBEX_RSP_FORBIDDEN: /* 0x43 */
+		d(g_printerr ("Error: RSP_FORBIDDEN\n"));
 		return GNOME_VFS_ERROR_NOT_PERMITTED;
 
 	case OBEX_RSP_NOT_FOUND: /* 0x44 */
+		d(g_printerr ("Error: RSP_NOT_FOUND\n"));
 		return GNOME_VFS_ERROR_NOT_FOUND;
 
-	case 0x48: /* REQUEST TIME OUT */
+	case OBEX_RSP_REQUEST_TIME_OUT: /* 0x48 */
+		d(g_printerr ("Error: RSP_REQUEST_TIME_OUT\n"));
 		return GNOME_VFS_ERROR_IO;
 
-	case 0x46: /* NOT_ACCEPTABLE */
+	case OBEX_RSP_NOT_ACCEPTABLE: /* 0x46 */
+		d(g_printerr ("Error: RSP_NOT_ACCEPTABLE\n"));
 		return GNOME_VFS_ERROR_NOT_PERMITTED;
 		
+	case OBEX_RSP_REQ_ENTITY_TOO_LARGE: /* 0x4d */
+		d(g_printerr ("Error: RSP_REQ_ENTITY_TOO_LARGE\n"));
+		return GNOME_VFS_ERROR_TOO_BIG;
+		
 	case OBEX_RSP_NOT_IMPLEMENTED:	/* 0x51 */
+		d(g_printerr ("Error: RSP_NOT_IMPLEMENTED\n"));
 		return GNOME_VFS_ERROR_NOT_SUPPORTED;
 
+	case OBEX_RSP_DATABASE_FULL: /* 0x60 */
+		d(g_printerr ("Error: RSP_DATABASE_FULL\n"));
+		return GNOME_VFS_ERROR_NO_SPACE;
+		
 	case OBEX_RSP_DATABASE_LOCKED: /* 0x61 */
+		d(g_printerr ("Error: RSP_DATABASE_LOCKED\n"));
 		return GNOME_VFS_ERROR_NOT_PERMITTED;
+
 		
 	default:
 		break;
